// ************************************************************* 
// Programmbeispiel: 	Lichtmessung mit TSL235
// 						Messung beruht auf Frequenzmessung mit automatischer Bereichswahl
//						Automatische Umschaltung erfolgt bei 65536 Hz bzw. 60000 Hz
//						Das erforderliche Tor-Signal wird von Timer0 (10ms) abgeleitet
//						Gezhlt wird mit Timer2. 					                           
// 						Datum: 6.11.2005: mpa										 
//  
//	Pulse                 |||||||||||||||||||||||||||||||||||| in Timer2 gezaehlt
//						          __________________        
//  Tor zur Messung     _________|                  |_______ von Timer1 abgeleitet
                                   

#include<reg517a.h>
#include<stdio.h>
#include<lcd.h>



// Globale variablen (beginnen mit g...)
char gTXT[25];							// Allgemeiner Textbuffer
int g10ms;								// Zaehlt die 10ms Ueberlaufe

// Deklaration der Funktionen


main()
{
unsigned long timer2val;						// 32 bit Ergebniss
int maxcount;									// Maximale Anzahl der 10ms Durchlaufe fuer autoscale

init_lcd();
blank_lcd();
maxcount = 100;									// Beginne mit 100*10ms = 1Sekunde Zaehldauer

TMOD	=0x01;									// Timer0 in moduls 01: 16 zaehler
g10ms 	=0;										// 10ms Zaehler  bei Null beginnen
TL0 	=0xF6;									// 65536 - 9994 = 55542 vorsetzen = 0xD8F6
TH0 	=0xD8;									// Naehere Beschreibung siehe Timer0 interrupt	
TH2 	=0;										// Timer 2 High byte loeschen
TL2 	=0;										// Timer 2 Low  byte loeschen		
TR0		=1;										// Timer0 starten
T2CON	=0x02; 									// Timer2 starten im Counter mode ( von pin P1.7 getaktet )

TF2		=0;										// Timer2 Ueberlaufbit loeschen
ET0		=1;										// Timer0 int freigeben
EAL		=1;										// Globale Interrupt Freigabe


while(1)										// Endlosschleife
	{
	while( g10ms < maxcount )	{ ; }			// Warte, geforderte Zaehldauer verstrichen
	T2CON = 0x00;								// Timer 2 stoppen
	TR0	  = 0;									// Timer 0 stoppen
	timer2val = 256*TH2 + TL2;					// 16 bit Wert aus T2L und T2H zusammensetzen
	if(maxcount == 10)							// Wenn Zaehldauer 100ms war, Ergebniss * 10 
		{
		timer2val = timer2val * 10;
		}

	if		(TF2 == 1 )							// Falls waehrend Zaehlung Ueberlauf auftrat Zaehldauer verkleinern
		{
		maxcount = 10;							// Zaehldauer auf 10*10ms = 100ms verringern und neustarten
		TF2 = 0;								// Ueberlaufflag wieder loeschen
		}
	else if ( timer2val < 60000	)				// Falls Zaehlerstand zu klein war Zaehldauer erhoehen
		{
		sprintf(gTXT,"Messzeit=   %4dms",maxcount*10);	// Ausgabe der Messzeit
		print_lcd(1,1,gTXT);					// Auf LCD ausgeben
		sprintf(gTXT,"Frequenz= %6luHz",timer2val);// In Textbuffer schreiben
		print_lcd(2,1,gTXT);					// Auf LCD ausgeben
		maxcount = 100;							// Zaehldauer auf 100*10ms = 100ms erhoehen
		}
	else
		{
		sprintf(gTXT,"Messzeit=   %4dms",maxcount*10);	// Ausgabe der Messzeit
		print_lcd(1,1,gTXT);					// Auf LCD ausgeben
		sprintf(gTXT,"Frequenz= %6luHz",timer2val);// In Textbuffer schreiben
		print_lcd(2,1,gTXT);					// Auf LCD ausgeben
		}

	g10ms 	=0;								// 10ms Zaehler  bei Null beginnen
	TL0 	=0xF6;							// 65536 - 9994 = 55542 vorsetzen = 0xD8F6
	TH0 	=0xD8;							// Naehere Beschreibung siehe Timer0 interrupt	
	TH2 	=0;								// Timer 2 High byte loeschen
	TL2 	=0;								// Timer 2 Low  byte loeschen		
	TR0		=1;								// Timer0 starten
	T2CON	=0x02; 							// Timer2 starten im Counter mode ( von pin P1.7 getaktet )
	}
}


// Interrupt timer 0 wird alle 10ms ausgeloest:
// Timertakt = fclk/12 = 12MHz/12 = 1MHz (1us)
// Einsprung und Setzten des Nachladewertes dauert 6 cycles (LCALL,LJMP,POP ACC)
// d.h. Zaehler 10000 - 6 = 9994 Takte zaehlen lassen
// d.h. Zahler auf 65536 - 9994 = 55542 vorsetzen = 0xD8F6

void INTTIM0(void) interrupt 1		
{
TL0 = 0xF6;
TH0 = 0xD8;									// Zaehler jetzt nachgeladen
g10ms++;									// 10ms Variable erhhen							
}


